# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - BlogInclude macro

    adapted from Include Macro for some Blog-features from Ulrich Norbisrath

    This macro includes the formatted content of the given page(s). See

        http://purl.net/wiki/moinmaster/HelpOnMacros/Include

    for detailed docs.

    Added to this doc are the following possible parameters:
     * leveldelta: the number of levels which should be added to the titles in the included pages (default: 0)
     * displaydate: if 1, display the date (pagename), default: 1
     * showcomment: if 1, show the comments for each page (created with PageComment2 or PageComment3), default: 1
     * commenteditor: if 1, show the comment editor via the specified string (example: commenteditor=PageComment3), default: 0
     * shownumberofcomments: if 1, show the number of comments for the entry. default 0
     * commentpagelink: if 1, show a link to the page, where comments can be added (this will jump to the anchors generated through commenteditor). default: 0
    
    @copyright: 2008 by Ulrich Norbisrath (ulno, http://ulno.net)
    @copyright: 2000-2004 by Jrgen Hermann <jh@web.de>
    @copyright: 2000-2001 by Richard Jones <richard@bizarsoftware.com.au>
    @license: GNU GPL, see COPYING for details.
"""

#Dependencies = ["pages"] # included page
Dependencies = ["time"] # works around MoinMoinBugs/TableOfContentsLacksLinks

import re, StringIO
from MoinMoin import wikiutil
from MoinMoin.Page import Page
from MoinMoin.util import web

_sysmsg = '<p><strong class="%s">%s</strong></p>'


_title_re = r"^(?P<heading>\s*(?P<hmarker>=+)\s.*\s(?P=hmarker))$"

def extract_titles(body, title_re):
    titles = []
    for title, _ in title_re.findall(body):
        h = title.strip()
        level = 1
        while h[level:level+1] == '=': level = level+1
        depth = min(5,level)
        title_text = h[level:-level].strip()
        titles.append((title_text, level))
    return titles

def execute(macro, text, title_re=re.compile(_title_re, re.M), called_by_toc=0):
    request = macro.request
    _ = request.getText

    # return immediately if getting links for the current page
    if request.mode_getpagelinks:
        return ''

    # parse and check arguments
    class Params:
	pagename = ""
	heading = ""
	level = 1
	pfrom = ""
	to = ""
	sort = ""
	items = ""
	skipitems = 0
	titlesonly = 0
	editlink = 0
	leveldelta = 0
	displaydate = 1
	showcomments = 1
	commenteditor = 0
	shownumberofcomments = 0
	commentpagelink = 0
    for label in dir(Params):
	if not re.match("^_",label):
	    result = re.search( r"[,\s^]*\b" + label + r"\s*=\s*([^=,]*)", text)
	    if result != None:
		val = result.groups()[0]
		if val[0] == "\"" or val[0] == "'":
		    result = re.search( r"[,\s^]*\b" + label + r"\s*=\s*" \
			+ val[0] + r"([^" + val[0] + r"]*)"  + val[0], text )
		    if result != None:
			val = result.groups()[0]
		setattr( Params, label, val )

    #if not args:
    #    return (_sysmsg % ('error', _('Invalid include arguments "%s"!')) % (text,))

    # prepare including page
    result = []
    print_mode = macro.form.has_key('action') and macro.form['action'][0] in ("print", "format")
    this_page = macro.formatter.page
    if not hasattr(this_page, '_macroInclude_pagelist'):
        this_page._macroInclude_pagelist = {}

    # get list of pages to include
#    inc_name = wikiutil.AbsPageName(request, this_page.page_name, Params.pagename)
    inc_name = wikiutil.AbsPageName(this_page.page_name, Params.pagename)
    pagelist = [inc_name]
    if inc_name.startswith("^"):
        try:
            inc_match = re.compile(inc_name)
        except re.error:
            pass # treat as plain page name
        else:
            # Get user filtered readable page list
            pagelist = request.rootpage.getPageList(filter=inc_match.match)

    # sort and limit page list
    pagelist.sort()
    sort_dir = Params.sort
    if sort_dir == 'descending':
        pagelist.reverse()
    max_items = Params.items
    if max_items:
        pagelist = pagelist[:int(max_items)]

    skipitems = 0
    if Params.skipitems:
        skipitems = int(Params.skipitems)
    titlesonly = Params.titlesonly
    editlink = Params.editlink

    # Start bulletlist, if titlesonly
    if titlesonly:
        result.append(macro.formatter.bullet_list(1))
    # iterate over pages
    for inc_name in pagelist:
        if not request.user.may.read(inc_name):
            continue
        if this_page._macroInclude_pagelist.has_key(inc_name):
            result.append(u'<p><strong class="error">Recursive include of "%s" forbidden</strong></p>' % (inc_name,))
            continue
        if skipitems:
            skipitems -= 1
            continue
        fmt = macro.formatter.__class__(request, is_included=True)
        fmt._base_depth = macro.formatter._base_depth
        inc_page = Page(request, inc_name, formatter=fmt)
        if not inc_page.exists():
            continue
        inc_page._macroInclude_pagelist = this_page._macroInclude_pagelist

        body = inc_page.get_raw_body() + '\n'
	# compute leveldelta
	leveldelta = max(min(int(Params.leveldelta),5),0)
	blog_title_re = re.compile(r"^\s*(={1,%s})([^=]*)(=+)\s*$"\
	    %(5-leveldelta), re.M)
	# add the pagename to the headline
	shortname = re.search(r"([^/]*)$",inc_name).group()
	if int(Params.displaydate):
	    body = blog_title_re.sub( r"\1 %s:\2\3"% \
		shortname, body, count=1)
	# add the level-delta to all headlines
	if leveldelta:
	    body = blog_title_re.sub( "="*leveldelta + r"\1\2\3" + "="*leveldelta, body )
	if not titlesonly:
	    # add an anchor to the start
	    body = "<<Anchor(%s)>>\n"%shortname + body
	    # append edit links
	    editcommentline = ""
	    if int(Params.commentpagelink):
		editcommentline += "[[../EditBlog#%s|comment]] / [[%s|edit]]    "%(shortname,inc_name)
	    if int(Params.shownumberofcomments):
		editcommentline += "number of comments: <<PageComment3(pagename=%s,countonly=1,articleview=1,tablewidth=100%%)>>"%inc_name
	    if editcommentline:
		body += "<<BR>>~-%s-~"%editcommentline
	    #body += " *** ce: %s sc: %s ***"%(str(Params.commenteditor),str(Params.showcomments)) # sebug
	    if int(Params.commenteditor) and int(Params.showcomments):
		body += "<<BR>><<PageComment3(pagename=%s,articleview=1,tablewidth=100%%,commentfirst=1,nosmiley=1,notify=1)>>\n"%inc_name
	    else:
    		if int(Params.commenteditor):
		    body += "<<BR>><<PageComment3(pagename=%s,inputonly=1,nosmiley=1,notify=1)>>\n"%inc_name
		if int(Params.showcomments):
		    body += "<<PageComment3(pagename=%s,commentonly=1,articleview=1,tablewidth=100%%)>>\n"%inc_name
        
	# check for "from" and "to" arguments (allowing partial includes)
        from_pos = 0
        to_pos = -1
        from_re = Params.pfrom
        if from_re:
            try:
                from_match = re.compile(from_re, re.M).search(body)
            except re.error, e:
                ##result.append("*** fe=%s ***" % e)
                from_match = re.compile(re.escape(from_re), re.M).search(body)
            if from_match:
                from_pos = from_match.end()
            else:
                result.append(_sysmsg % ('warning', 'Include: ' + _('Nothing found for "%s"!')) % from_re)
        to_re = Params.to
        if to_re:
            try:
                to_match = re.compile(to_re, re.M).search(body, from_pos)
            except re.error:
                to_match = re.compile(re.escape(to_re), re.M).search(body, from_pos)
            if to_match:
                to_pos = to_match.start()
            else:
                result.append(_sysmsg % ('warning', 'Include: ' + _('Nothing found for "%s"!')) % to_re)

	if titlesonly:
            newbody = []
            levelstack = []
            for title, level in extract_titles(body[from_pos:to_pos], title_re):
                if levelstack:
                    if level > levelstack[-1]:
                        result.append(macro.formatter.bullet_list(1))
                        levelstack.append(level)
                    else:
                        while levelstack and level < levelstack[-1]:
                            result.append(macro.formatter.bullet_list(0))
                            levelstack.pop()
                        if not levelstack or level != levelstack[-1]:
                            result.append(macro.formatter.bullet_list(1))
                            levelstack.append(level)
                else:
                    #result.append(macro.formatter.bullet_list(1))
                    levelstack.append(level)
                result.append(macro.formatter.listitem(1))
                #result.append(inc_page.link_to(request, title)) # TODO: make linking possible again
                result.append(title)
                result.append(macro.formatter.listitem(0))
            while len(levelstack)>1:
                result.append(macro.formatter.bullet_list(0))
                levelstack.pop()
            continue # next iteration

        if from_pos or to_pos != -1:
            inc_page.set_raw_body(body[from_pos:to_pos], modified=True)
	else:
	    inc_page.set_raw_body(body, modified=True)
        ##result.append("*** f=%s t=%s ***" % (from_re, to_re))
    	##result.append("*** f=%d t=%d ***" % (from_pos, to_pos))

        if called_by_toc:
            result.append(inc_page.get_raw_body())
            continue

        if not hasattr(request, "_Include_backto"):
            request._Include_backto = this_page.page_name
        
        # do headings
        level = None
        if Params.heading:
            heading = Params.heading
            level = 1
            if Params.level:
                level = int(Params.level)
            if print_mode:
                result.append(macro.formatter.heading(1, level) +
                              macro.formatter.text(heading) +
                              macro.formatter.heading(0, level))
            else:
                import sha
                from MoinMoin import config
                # this heading id might produce duplicate ids,
                # if the same page is included multiple times
                # Encode stuf we feed into sha module.
                pntt = (inc_name + heading).encode(config.charset)
                hid = "head-" + sha.new(pntt).hexdigest()
                request._page_headings.setdefault(pntt, 0)
                request._page_headings[pntt] += 1
                if request._page_headings[pntt] > 1:
                    hid += '-%d'%(request._page_headings[pntt],)
                result.append(
                    #macro.formatter.heading(1, level, hid,
                    #    icons=edit_icon.replace('<img ', '<img align="right" ')) +
                    macro.formatter.heading(1, level, id=hid) +
                    inc_page.link_to(request, heading, css_class="include-heading-link") +
                    macro.formatter.heading(0, level)
                )

        # set or increment include marker
        this_page._macroInclude_pagelist[inc_name] = \
            this_page._macroInclude_pagelist.get(inc_name, 0) + 1

        # output the included page
        strfile = StringIO.StringIO()
        request.redirect(strfile)
        try:
            #cid = request.makeUniqueID("Include_%s" % wikiutil.quoteWikinameURL(inc_page.page_name))
            cid = "Include_%s" % wikiutil.quoteWikinameURL(inc_page.page_name)
            #inc_page.send_page(request, content_only=1, content_id=cid,
            inc_page.send_page(content_only=1, content_id=cid,
                               omit_footnotes=True)
            result.append(strfile.getvalue())
        finally:
            request.redirect()

        # decrement or remove include marker
        if this_page._macroInclude_pagelist[inc_name] > 1:
            this_page._macroInclude_pagelist[inc_name] = \
                this_page._macroInclude_pagelist[inc_name] - 1
        else:
            del this_page._macroInclude_pagelist[inc_name]

        # if no heading and not in print mode, then output a helper link
        if editlink and not (level or print_mode):
            result.extend([
                macro.formatter.div(1, css_class="include-link"),
                inc_page.link_to(request, '[%s]' % (inc_name,), css_class="include-page-link"),
                inc_page.link_to(request, '[%s]' % (_('edit'),), css_class="include-edit-link", querystr={'action': 'edit', 'backto': request._Include_backto}),
                macro.formatter.div(0),
            ])
        # XXX page.link_to is wrong now, it escapes the edit_icon html as it escapes normal text
    
    # close buletlist if titles only
    if titlesonly:
        result.append(macro.formatter.bullet_list(0))

    # return include text
    return ''.join(result)

# vim:ts=4:sw=4:et
